@@ -6,10 +6,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.lex.c
-    Revision:  1.5
-        Date:  01 Feb 1995
-     Modtime:  05:22:42
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -424,6 +424,7 @@ void GetNumber (PINPUTBUF pibIn, PSTR szLex, PINT piLexType)
 
 void NextLex (PINPUTBUF pibIn, PSTRLEX szLex, PINT piLexType)
 {
+  static char vszEqnPunct[] = "+-/*()><?:,!=";
   char c;
   BOOL fDone = FALSE;
 
@@ -455,7 +456,7 @@ void NextLex (PINPUTBUF pibIn, PSTRLEX szLex, PINT piLexType)
         if (IsSign(c) && !*piLexType) { /* Unary +/- for identifier */
           szLex[0] = c;
           szLex[1] = '\0';
-          *piLexType = LX_PUNCT;
+          *piLexType = LX_EQNPUNCT;
         } /* if */
       } /* if */
 
@@ -465,6 +466,17 @@ void NextLex (PINPUTBUF pibIn, PSTRLEX szLex, PINT piLexType)
         GetString (pibIn, szLex);
       } /* if */
 
+      else if (strchr(vszEqnPunct, c)) {
+        *piLexType = LX_EQNPUNCT;
+        szLex[0] = *pibIn->pbufCur++;
+        if((c = *pibIn->pbufCur) != '=')
+          szLex[1] = '\0';
+        else if (szLex[0] == '!' || szLex[0] == '<' ||
+                 szLex[0] == '>' || szLex[0] == '=')
+          szLex[1] = *pibIn->pbufCur++;
+          szLex[2] = '\0';
+      }
+
       else { /* Is other punctuation -- Take one char */
         *piLexType = LX_PUNCT;
         szLex[0] = *pibIn->pbufCur++;
@@ -580,7 +592,7 @@ int GetPunct (PINPUTBUF pibIn, PSTR szLex, char chPunct)
   int iType;
 
   NextLex (pibIn, szLex, &iType);
-  return ((iType == LX_PUNCT) && szLex[0] == chPunct);
+  return ((iType == LX_PUNCT || iType == LX_EQNPUNCT) && szLex[0] == chPunct);
 
 } /* GetPunct */
 
