@@ -1,10 +1,10 @@
 /* getopt.c
 
    -- Revisions -----
-     Logfile:  SCCS/s.getopt.c
-    Revision:  1.12
-        Date:  28 Jan 1995
-     Modtime:  03:43:26
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -110,7 +110,8 @@ static enum
    whose names are inconsistent.
 */
 
-/* char *getenv (); */
+/* char *getenv (); FB: this is not a proper declaration, getenv is already
+   in stdlib anyway */
 
 static char *
 my_index (char *string, int chr)
