@@ -6,10 +6,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.lex.c
-    Revision:  1.9
-        Date:  25 Aug 1995
-     Modtime:  11:11:58
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -742,15 +742,15 @@ BOOL GetFuncArgs (PINPUTBUF pibIn,
   for (i = 0; i < nArgs && bOK; i++, szArgs += MAX_LEX) {
     if (i)
       if (!(bOK = GetOptPunct (pibIn, szArgs, ','))) {
-    *(szArgs+1) = ',';
-    ReportError(pibIn, RE_EXPECTED, szArgs, NULL);
-    break;                /* Error: Stop getting args */
+        *(szArgs+1) = ',';
+        ReportError(pibIn, RE_EXPECTED, szArgs, NULL);
+        break;                /* Error: Stop getting args */
       }
 
     NextLex (pibIn, szArgs, &iType);
     if (!(bOK &= (iType & rgiArgTypes[i]) > 0))
-      ReportError(pibIn, RE_LEXEXPECTED,
-              vrgszLexTypes[rgiArgTypes[i]], szArgs);
+      ReportError (pibIn, RE_LEXEXPECTED,
+                   vrgszLexTypes[rgiArgTypes[i]], szArgs);
   } /* for */
 
   if (!(bOK = GetPunct (pibIn, szPunct, CH_RPAREN))) {
@@ -759,6 +759,7 @@ BOOL GetFuncArgs (PINPUTBUF pibIn,
   } /* if */
 
   return (bOK);
+
 } /* GetFuncArgs */
 
 
