@@ -1,4 +1,4 @@
-/* lex.h
+/* lexerr.h
 
    written by Don Maszle
    13 October 1991
@@ -7,9 +7,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.lexerr.h
-    Revision:  1.4
-        Date:  28 Jan 1995
-     Modtime:  03:44:05
+    Revision:  1.5
+        Date:  23 Nov 1996
+     Modtime:  12:38:49
       Author:  @a
    -- SCCS  ---------
 
@@ -18,16 +18,18 @@
 
 #ifndef _LEXERR_H_
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Inclusions  */
 
 #include "hungtype.h"
 #include "lex.h"
+#include "sim.h"
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Prototypes */
 
 void ReportError (PINPUTBUF, WORD, PSTR, PSTR);
+void ReportRunTimeError (PANALYSIS, WORD, ...);
 
 #define _LEXERR_H_
 #endif
