@@ -6,10 +6,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.list.c
-    Revision:  1.7
-        Date:  28 Jan 1995
-     Modtime:  03:43:29
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -152,3 +152,27 @@ int ForAllList (PLIST plist, PFI_FORLISTCALLBACK pfiCallback, PVOID pUserInfo)
   return (iTotal);
 
 } /* ForAllList */
+
+/* -----------------------------------------------------------------------------
+   ForAllList3
+
+   Like ForAllList, but takes three user arguments
+   Included temporarily to avoid rewriting a lot of code
+*/
+
+void ForAllList3(PLIST plist, PFI_FORLISTCALLBACK3 pfiCallback,
+                 PVOID pUserInfo1, PVOID pUserInfo2, PVOID pUserInfo3)
+{
+  PLISTELEM ple;
+
+  if (!plist || !pfiCallback)
+    return;
+
+  ple = plist->pleHead;
+
+  while (ple) {
+    (*pfiCallback) (ple->pData, pUserInfo1, pUserInfo2, pUserInfo3);
+    ple = ple->pleNext; /* Increment to next elem */
+  }
+
+} /* ForAllList3 */
