@@ -6,10 +6,10 @@
    Copyright (c) 1995.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.mac.h
-    Revision:  1.2
-        Date:  18 Jan 1996
-     Modtime:  10:21:48
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -25,7 +25,7 @@
 /* Private */
 
 #ifdef _MACOS_
-
+/*
 void AdjustMenus(void);
 void FileError(Str255 s, Str255 f);
 void HandleEvent(void);
@@ -41,7 +41,7 @@ void ShowSplash (void);
 
 
 static void Enable (MenuHandle menu, short item, short ok);
-
+*/
 #endif
 
 /* End */
