@@ -6,10 +6,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.modelu.c
-    Revision:  1.16
-        Date:  15 Apr 1996
-     Modtime:  05:57:24
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -31,8 +31,8 @@
 
 #define _MODEL_DEF_FILE
 
+#include "lexerr.h"
 #include "modelu.h"
-#include "sim.h" /* only for matType1 ! */
 #include "strutil.h"
 #include "hungtype.h"
 
@@ -52,9 +52,9 @@ extern int vnModelVars;
 extern int vnInputs;
 extern int vnParms;
 
-extern double vrgModelVars[];    /* States and Outputs */
-extern IFN    vrgInputs[];    /* Input Function records */
-extern VM     vrgvmGlo[];    /* Global variable map */
+extern double     vrgModelVars[]; /* States and Outputs */
+extern IFN        vrgInputs[];    /* Input Function records */
+extern VMMAPSTRCT vrgvmGlo[];     /* Global variable map */
 
 /* WARNING! ===== GLOBAL VARIABLE ! */
 
@@ -70,7 +70,7 @@ BOOL vbModelReinitd = FALSE;    /* Model has been initialized */
 #endif
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    FixupDependentInputs
 
    Fixes input definitions dependent on model variables.
@@ -116,13 +116,23 @@ void FixupDependentInputs()
       vrgInputs[i].dDecay = GetVarValue (vrgInputs[i].hDecay);
 
     /* NDoses dependencies */
-    if (vrgInputs[i].iType == IFN_NDOSES) /* FB 12/10/96 */
-    for (j = 0; j < vrgInputs[i].nDoses; j++) {
-      if (vrgInputs[i].rghT0s[j])
-        vrgInputs[i].rgT0s[j] = GetVarValue (vrgInputs[i].rghT0s[j]);
-      if (vrgInputs[i].rghMags[j])
-        vrgInputs[i].rgMags[j] = GetVarValue (vrgInputs[i].rghMags[j]);
-    }
+    if (vrgInputs[i].iType == IFN_NDOSES)  /* FB 12/10/96 */
+      for (j = 0; j < vrgInputs[i].nDoses; j++) {
+        if (vrgInputs[i].rghMags[j])
+          vrgInputs[i].rgMags[j] = GetVarValue (vrgInputs[i].rghMags[j]);
+        if (vrgInputs[i].rghT0s[j])
+          vrgInputs[i].rgT0s[j] = GetVarValue (vrgInputs[i].rghT0s[j]);
+
+        /* check that the times are in order */
+        if (j > 0) {
+          if (vrgInputs[i].rgT0s[j] <= vrgInputs[i].rgT0s[j-1]) {
+            printf ("\nError: unordered pair of times (%g, %g) in NDoses "
+                    "statement - Exiting\n", vrgInputs[i].rgT0s[j-1], 
+                    vrgInputs[i].rgT0s[j]);
+            exit (0);
+          }
+        }
+      }
 
     /* Fix exponential exposure times for efficiency */
 
@@ -134,8 +144,8 @@ void FixupDependentInputs()
 
     /* Assure that input has an off zone at end of period */
 
-    if (vrgInputs[i].dTexp != 0.0 && 
-        vrgInputs[i].dT0 + vrgInputs[i].dTexp >= vrgInputs[i].dTper)
+    if (vrgInputs[i].dTexp != 0.0
+        && vrgInputs[i].dT0 + vrgInputs[i].dTexp >= vrgInputs[i].dTper)
       vrgInputs[i].dTexp = vrgInputs[i].dTper - vrgInputs[i].dT0 - 1e-6;
 
     /* For doses and spikes allow scaling in Scale{} by use of dMag */
@@ -144,17 +154,17 @@ void FixupDependentInputs()
       vrgInputs[i].dMag = 1.0; /* Initially no scaling */
 
     /* If the input is ill-defined, turn it off */
-
     if (vrgInputs[i].iType != IFN_NDOSES && vrgInputs[i].iType != IFN_CONSTANT
         && vrgInputs[i].iType != IFN_SPIKES) {
       if (vrgInputs[i].dTexp == 0.0
-      || vrgInputs[i].dT0 > vrgInputs[i].dTper
-      || vrgInputs[i].dT0 < 0.0
-      || vrgInputs[i].dTper < 0.0 )
+          || vrgInputs[i].dT0 > vrgInputs[i].dTper
+          || vrgInputs[i].dT0 < 0.0
+          || vrgInputs[i].dTper < 0.0 )
         vrgInputs[i].dMag = 0.0;
     } /* if */
 
   } /* for */
+
 } /* FixupDependentInputs */
 
 
@@ -284,9 +294,13 @@ BOOL UpdateSpikes (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 
 void PostUpdateSpikes (PDOUBLE pdTime)
 {
-  matType1 rgDerivTemp; /*   Derivatives are NOT affected */
+  static PDOUBLE rgDerivTemp; /* Derivatives are NOT affected */
   int i;
 
+  if (!(rgDerivTemp))
+    if ( !(rgDerivTemp = InitdVector (GetNModelVars ())))
+      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "PostUpdateSpikes", NULL);
+
   CalcDeriv (GetModelVector(), rgDerivTemp, pdTime);
   for (i = 0; i < vnInputs; i++) /* Turn spikes off */
     if (vrgInputs[i].iType == IFN_SPIKES
@@ -295,6 +309,7 @@ void PostUpdateSpikes (PDOUBLE pdTime)
       vrgInputs[i].dVal = 0.0;
       vrgInputs[i].iDoseCur++; /* Advance to next spike */
     } /* if */
+
 } /* PostUpdateSpikes */
 
 
@@ -402,11 +417,11 @@ void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
 /* -----------------------------------------------------------------------------
    GetVarPtr
 
-   Returns a PVM to the structure of the variable
+   Returns a PVMMAPSTRCT to the structure of the variable
    specified by szName, or NULL if it does not exist.
 */
 
-PVM GetVarPtr (PVM pvm, PSTR szName)
+PVMMAPSTRCT GetVarPtr (PVMMAPSTRCT pvm, PSTR szName)
 {
   while (*pvm->szName && MyStrcmp (szName, pvm->szName))
     pvm++;
@@ -425,7 +440,7 @@ PVM GetVarPtr (PVM pvm, PSTR szName)
 
 HVAR GetVarHandle (PSTR szName)
 {
-  PVM pvm = GetVarPtr (vrgvmGlo, szName);
+  PVMMAPSTRCT pvm = GetVarPtr (vrgvmGlo, szName);
 
   return (pvm ? pvm->hvar : ID_NULL);
 } /* GetVarHandle */
@@ -486,7 +501,7 @@ int GetVarType (HVAR hvar)
 char *GetVarName (HVAR hvar)
 {
 static char szInvalid[] = "InvalidVariable?";
-  PVM pvm = vrgvmGlo;
+  PVMMAPSTRCT pvm = vrgvmGlo;
 
   while (*pvm->szName && hvar != pvm->hvar)
     pvm++;
@@ -518,7 +533,7 @@ double GetVarValue (HVAR hvar)
 
     case ID_PARM:
     {
-      PVM pvm;
+      PVMMAPSTRCT pvm;
 
       pvm = &vrgvmGlo[HINDEX(hvar)];
       dReturn = *(PDOUBLE) pvm->pVar;
@@ -631,7 +646,7 @@ BOOL SetVar (HVAR hvar, double dVal)
 
     case ID_PARM:
     {
-      PVM pvm = &vrgvmGlo[HINDEX(hvar)];
+      PVMMAPSTRCT pvm = &vrgvmGlo[HINDEX(hvar)];
       *(PDOUBLE) pvm->pVar = dVal;
       break;
     } /* ID_PARM: */
@@ -719,25 +734,27 @@ void CalcInputs (PDOUBLE pdTime)
     && vrgInputs[i].bOn) { /* Exposure must be on */
 
       switch (vrgInputs[i].iType) {
-        case IFN_CONSTANT:
-          break; /* dVal does not change */
+    case IFN_CONSTANT:
+      break; /* dVal does not change */
 
-       case IFN_PERDOSE:
-       case IFN_PERRATE: /* Does not change step-wise */
-         break;
+    case IFN_PERDOSE:
+    case IFN_PERRATE: /* Does not change step-wise */
 
-       case IFN_PEREXP: /* Calc current exp() */
-         vrgInputs[i].dVal = vrgInputs[i].dMag * 
-                             exp ((vrgInputs[i].dTStartPeriod - *pdTime) *
-                             vrgInputs[i].dDecay);
-         break;
+      break;
 
-       case IFN_SPIKES:
-       case IFN_NDOSES:
-         if (vrgInputs[i].iDoseCur < vrgInputs[i].nDoses)
-           vrgInputs[i].dVal = vrgInputs[i].rgMags[vrgInputs[i].iDoseCur] *
-                               vrgInputs[i].dMag; /* Scaled */
-         break;
+    case IFN_PEREXP: /* Calc current exp() */
+      vrgInputs[i].dVal =
+          vrgInputs[i].dMag
+          * exp ((vrgInputs[i].dTStartPeriod - *pdTime)
+             * vrgInputs[i].dDecay);
+      break;
+
+        case IFN_SPIKES:
+    case IFN_NDOSES:
+      if (vrgInputs[i].iDoseCur < vrgInputs[i].nDoses)
+        vrgInputs[i].dVal = vrgInputs[i].rgMags[vrgInputs[i].iDoseCur] *
+                            vrgInputs[i].dMag; /* Scaled */
+      break;
 
       } /* switch */
     } /* if */
@@ -756,7 +773,7 @@ void DumpSymbolTable (char *szFilename)
 {
   static char szStderr[] = "<stdout>";
   FILE *pfile;
-  PVM pvm = &vrgvmGlo[0];
+  PVMMAPSTRCT pvm = &vrgvmGlo[0];
 
   if (szFilename)
     pfile = fopen(szFilename, "a");
@@ -791,3 +808,23 @@ void DumpSymbolTable (char *szFilename)
   if (szFilename != szStderr) fclose (pfile);
 
 } /* DumpSymbolTable */
+
+
+/* ----------------------------------------------------------------------------
+   GetStateHandles
+
+   Sets up an array of state variable handles
+*/
+
+void GetStateHandles (HVAR *phvar)
+{
+  int i = 0;
+  VMMAPSTRCT *pvm = vrgvmGlo;
+  while(pvm->szName) {
+    if(IsState(pvm->hvar))
+      phvar[i++] = pvm->hvar;
+    ++pvm;
+  };
+
+} /* GetStateHandles */
+
