@@ -6,10 +6,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.modelu.h
-    Revision:  1.5
-        Date:  28 Jan 1995
-     Modtime:  03:43:45
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -51,7 +51,7 @@ typedef struct tagVM {
     PSTR szName;     /* Name of the variable */
     PVOID pVar;      /* Ptr to C variable */
     HVAR hvar;       /* Handle to variable: ID_TYPE | index */
-} VM, *PVM; /* Variable Map element */
+} VMMAPSTRCT, *PVMMAPSTRCT; /* Variable Map element */
 
 
 /* -----------------------------------------------------------------------------
@@ -70,7 +70,8 @@ typedef struct tagVM {
 void FixupDependentInputs(void);
 
 void GetStartPeriods (PDOUBLE pdTime);
-PVM  GetVarPtr (PVM pvm, PSTR szName);
+void GetStateHandles (HVAR *phvar);
+PVMMAPSTRCT  GetVarPtr (PVMMAPSTRCT pvm, PSTR szName);
 int  GetVarType (HVAR hvar);
 
 void PostUpdateSpikes (PDOUBLE pdTime);
