@@ -7,10 +7,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.random.c
-    Revision:  1.16
-        Date:  23 Aug 1996
-     Modtime:  05:37:11
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -26,6 +26,7 @@
    BinomialRandom(p, n)               -- Binomial of n trials, P(each) = p
    Chi2Random(dof)                    -- Chi-squared w/dof degrees of freedom
    ExpRandom(beta)                    -- Exponential of inverse scale beta
+   InvGGammaRandom (alpha, beta)      -- General inverse gamma variate
    GammaRandom (alpha)                -- Gamma variate
    GGammaRandom (alpha, beta)         -- General gamma variate
    LogNormalRandom (m, s)             -- exp (Normal)
@@ -67,6 +68,8 @@
 #include <stdlib.h>
 
 #include "hungtype.h"
+#include "lex.h"
+#include "lexerr.h"
 #include "random.h"
 
 
@@ -79,7 +82,6 @@ typedef struct tagRANDREC {
 } RANDREC, *PRANDREC;
 
 
-#define SEED_DEFAULT 314159265.3589793
 #define SEED_MIN     1.0
 #define SEED_MAX     2147483646.0
 #define PI           3.1415926535897932384626433
@@ -126,7 +128,7 @@ void SetSeed (double dSeed)
 } /* SetSeed */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    InitRandom
 
    initializes the random generator with the given seed.
@@ -142,8 +144,8 @@ void InitRandom (double dSeed, int bWarmUp)
 {
   long i;
 
-  if (vbNoSeed
-      || dSeed != SEED_DEFAULT) /* Prevent nuking user's seed if not initd */
+  /* Prevent nuking user's seed if not initd */
+  if (vbNoSeed || dSeed != SEED_DEFAULT) 
     SetSeed (dSeed);
 
   if (bWarmUp) {
@@ -161,7 +163,7 @@ void InitRandom (double dSeed, int bWarmUp)
 } /* InitRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    Randoms
 
    An alternative random number generator, so you don't have to use
@@ -180,10 +182,10 @@ void InitRandom (double dSeed, int bWarmUp)
 
 double Randoms (void)
 {
-  #define a  16807.0
-  #define m  2147483647.0
-  #define q  127773.0   /* m Div a*/
-  #define r  2836.0     /* m Mod a*/
+#define a  16807.0
+#define m  2147483647.0
+#define q  127773.0   /* m Div a*/
+#define r  2836.0     /* m Mod a*/
 
   double hi, test;
 
@@ -200,15 +202,15 @@ double Randoms (void)
 
   return (vRandRec.seed / m);
 
-  #undef a
-  #undef m
-  #undef q
-  #undef r
+#undef a
+#undef m
+#undef q
+#undef r
 
 } /* Randoms */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    RandomShuffle
 
    assumes that the random number generator is Randoms ().
@@ -236,7 +238,7 @@ double RandomShuffle (void)
 } /* RandomShuffle */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    NormalRandom
 
    Returns a Normal random variate based on a unit variate,
@@ -273,7 +275,7 @@ double NormalRandom (double dMean, double dStdDev)
 } /* NormalRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    LogNormalRandom
 
    returns a variate such that the log of the variate is normally
@@ -283,29 +285,44 @@ double NormalRandom (double dMean, double dStdDev)
 double LogNormalRandom (double dMean, double dStdDev)
 {
 
-  return exp(NormalRandom((dMean), (dStdDev)));
+  if (dMean <= 0) {
+    char str[10];
+    sprintf(str, "%5.2e", dMean);
+    ReportRunTimeError(NULL, RE_BADLOGNORMALMEAN | RE_FATAL,
+                       "unknown", str, "LogNormalRandom");
+  }
+  else 
+    if (dStdDev <= 0) {
+      char str[10];
+      sprintf(str, "%5.2e", dStdDev);
+      ReportRunTimeError(NULL, RE_BADLOGNORMALSD | RE_FATAL,
+                         "unknown", str, "LogNormalRandom");
+    }
+
+  return exp (NormalRandom (log (dMean), log (dStdDev)));
 
 } /* LogNormalRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    TruncNormalRandom
 
    returns a truncated Normal variate in the range [a, b].
-   print a warning if the number of trials exceeds 25.
 */
 
 double TruncNormalRandom (double dMean, double dStdDev, double a, double b)
 {
   double X = 0.0;
-  int iter = 0;
+  int    iter = 0;
 
-  if (a >= b)
-    printf ("TruncNormalRandom: min >= max  [%g %g]\n", a, b);
+  if (a >= b) {
+    printf ("Error: TruncNormalRandom: min >= max  [%g %g]\n", a, b);
+    exit (0);
+  }
 
   else do {
     if(++iter == 25) {
-      printf("TruncNormalRandom: problem with range: ");
+      printf("Warning: TruncNormalRandom: problem with range: ");
       printf("min %g, max %g, ave %g, sd %g\n", a, b, dMean, dStdDev);
     }
     X = NormalRandom(dMean, dStdDev);
@@ -317,7 +334,7 @@ double TruncNormalRandom (double dMean, double dStdDev, double a, double b)
 } /* TruncNormalRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    TruncLogNormalRandom
 
    returns a truncated LogNormal variate in the range [a, b].
@@ -326,11 +343,16 @@ double TruncNormalRandom (double dMean, double dStdDev, double a, double b)
 double TruncLogNormalRandom (double dMean, double dStdDev, double a, double b)
 {
   double X = 0.0;
+  int    iter = 0;
 
   if (a >= b)
     printf ("TruncLogNormalRandom: min >= max  [%g %g]\n", a, b);
 
   else do {
+    if(++iter == 25) {
+      printf("TruncLogNormalRandom: problem with range: ");
+      printf("min %g, max %g, ave %g, sd %g\n", a, b, dMean, dStdDev);
+    }
     X = LogNormalRandom(dMean, dStdDev);
   }
   while (X < a || X > b);
@@ -340,19 +362,25 @@ double TruncLogNormalRandom (double dMean, double dStdDev, double a, double b)
 } /* TruncLogNormalRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    ExpRandom
 
    returns an exponential variate with inverse scale beta
-   
+
    Algorithm 3.2 from Ripley "Stochastic Simulations" Wiley 1987, p. 55.
 */
 
 double ExpRandom (double beta)
 {
 
+  if (beta <= 0) {
+    printf ("Error: negative or null scale for an exponential variate "
+            "- Exiting\n\n");
+    exit (0);
+  }
+
   return -log(Randoms()) / beta;
-  
+
 } /* ExpRandom */
 
 
@@ -365,19 +393,23 @@ double ExpRandom (double beta)
    If alpha < 1 uses algorithm 3.19 of Ripley; if alpha > 1 uses
    algorithm 3.20 of Ripley; if alpha is 1 uses returns an
    ExpRandom variate.
-   
+
    Reference:
    - Ripley, Stochastic Simulations, John Wiley and Sons, 1987, pp 88-90.
 */
 
 double GammaRandom (double alpha)
 {
-
-  #define E 2.71828182846
+#define E 2.718281828459
   static double aprev = 0.0, c1, c2, c3, c4, c5;
   double b, u1, u2, w, x;
-  
-  if (alpha < 1) {
+
+  if (alpha <= 0) {
+    printf ("Error: negative or null shape parameter for a gamma variate "
+            "- Exiting\n\n");
+    exit (0);
+  }
+  else if (alpha < 1) {
 
     b = (alpha + E) / E;
 
@@ -393,9 +425,9 @@ double GammaRandom (double alpha)
         if (pow(x, alpha - 1) >= Randoms()) return(x);
       }
     } while (1);
-    
+
   } /* end if alpha < 1 */
-  
+
   else {
     if (alpha > 1) {
 
@@ -410,7 +442,7 @@ double GammaRandom (double alpha)
         if (alpha > 2.5)
           c5 = 1.0 / sqrt(alpha);
       }
-      
+
       do {
         do {
           u1 = Randoms();
@@ -429,7 +461,7 @@ double GammaRandom (double alpha)
     else
       return ExpRandom(1.0);
   }
-  
+
   #undef E
 
 } /* GammaRandom */
@@ -445,21 +477,50 @@ double GammaRandom (double alpha)
 double GGammaRandom (double alpha, double beta)
 {
 
+  if (beta <= 0) {
+    printf ("Error: negative or null inverse scale for a gamma variate "
+            "- Exiting\n\n");
+    exit (0);
+  }
   return GammaRandom(alpha) / beta;
 
 } /* GGammaRandom */
 
 
+/* ----------------------------------------------------------------------------
+   InvGGammaRandom
+
+   Returns an inverse gamma distributed random variate with shaping parameter
+   alpha and scale parameter beta.
+   This just gets a general gamma variate and returns its inverse
+   See Gelman et al. "Bayesian Data Analysis"
+*/
+
+double InvGGammaRandom (double alpha, double beta)
+{
+
+  /* parameters will be checked in GGammaRandom */
+  if (beta <= 0) {
+    printf ("Error: negative or null scale for an inverse gamma variate "
+            "- Exiting\n\n");
+    exit (0);
+  }
+  
+  return beta / GammaRandom(alpha);
+
+} /* InvGGammaRandom */
+
+
 /* ----------------------------------------------------------------------------
    Chi2Random
 
    returns a chi-squared random variate, which is a gamma(dof/2, 1/2).
 */
 
-double Chi2Random(double dof)
+double Chi2Random (double dof)
 {
 
-  return (GGammaRandom(dof / 2.0, 0.5));
+  return (GGammaRandom (dof / 2.0, 0.5));
 
 } /* Chi2Random */
 
@@ -482,7 +543,8 @@ long PoissonRandom(double mu)
   long n = 0;
 
   if (mu <= 0) {
-    printf("Error in PoissonRandom: rate <= 0\n");
+    printf("Error: negative or null rate for a Poisson variate "
+           "- Exiting\n\n");
     exit(0);
   }
 
@@ -505,7 +567,7 @@ long PoissonRandom(double mu)
       alpha = beta * mu;
       k = log(c) - mu - log(beta);
     }
-    
+
     do {
       do {
         u1 = Randoms();
@@ -514,7 +576,7 @@ long PoissonRandom(double mu)
 
       n = (long)(x + 0.5);
       u2 = Randoms();
-      
+
       /* calculate log n factorial using Stirling's formula */
       lnfact = 0.918938533 - n + (n + 0.5) * log(n);
     } while (alpha - beta * x + log(u2 / pow((1 + exp(alpha - beta * x)), 2))
@@ -526,7 +588,7 @@ long PoissonRandom(double mu)
 } /* PoissonRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    BinomialRandom
 
    Return as a double floating-point number an integer value that is a random
@@ -538,29 +600,35 @@ long PoissonRandom(double mu)
 double BinomialRandom (double p, long n)
 {
   long j;
-  static long nOld=(-1);
+  static long nOld = (-1);
   double mean, em, g, angle, pTemp, bnl, sq, t, y;
-  static double pOld=(-1.0), pc, pLog, pcLog, en, gOld;
+  static double pOld = (-1.0), pc, pLog, pcLog, en, gOld;
+
+  if (p < 0 || p > 1 || n < 0) {
+    printf ("Error: negative or null parameters for a binomial variate "
+            "- Exiting\n\n");
+    exit (0);
+  }
 
   pTemp = ( p <= 0.5 ? p : 1.0 - p);
   mean = n * pTemp;  /* This is the mean of the deviate to be produced. */
 
-    /* Use the direct method while n is not too large.
-       This can require up to 25 calls to random */
+  /* Use the direct method while n is not too large.
+     This can require up to 25 calls to random */
 
   if (n < 25) {
     bnl = 0.0;
-    for (j=1; j<=n; j++)
+    for (j = 1; j <= n; j++)
       if (Randoms () < pTemp )
         bnl += 1.0;
-  } 
+  }
   else
     if (mean < 1.0) {
       /* if less than one event is expected out of 25 or more trials,then the
          distribution is quite accurately Poisson. Use direct method. */
       g = exp(-mean);
       t = 1.0;
-      for (j=0; j<=n; j++) {
+      for (j = 0; j <= n; j++) {
         t *= Randoms ();
         if (t < g) break;
       } /* for */
@@ -621,7 +689,7 @@ double BinomialRandom (double p, long n)
 } /* BinomialRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    Multinomial
 
    A procedure to compute return multinomial deviates.
@@ -640,7 +708,7 @@ void Multinomial (long n, int dim, double *p, double *x)
 
   sum = 1;
 
-  for (i=1; i<=dim; i++) {
+  for (i = 1; i <= dim; i++) {
     if (p[i]) {
       ptemp = p[i] / sum;
       x[i] = BinomialRandom (ptemp, n);
@@ -654,7 +722,7 @@ void Multinomial (long n, int dim, double *p, double *x)
 } /* Multinomial */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    Wishart
 
    samples a matrix according to the Wishart distribution by the method
@@ -667,19 +735,19 @@ void Multinomial (long n, int dim, double *p, double *x)
    triangular form; work (pointer to a work space, of length p*p).
 
    Triangular matrices are stored in order
-   0 1 3 
-     2 4 
+   0 1 3
+     2 4
        5 etc.
 */
 
 void WishartRandom (long n, long p, double *t, double *w, double *work)
-{ 
-  double eta, sum; 
+{
+  double eta, sum;
   long i, j, k, m, k1, k2, k3;
 
   printf ("WishartRandom not tested - Exiting...");
   exit(0);
-  
+
   /* generate random quantities for Bartlett's decomposition */
   for (j = 0, k = 0; j < p; j++) {
     for (i = 0; i < j; i++)
@@ -699,7 +767,7 @@ void WishartRandom (long n, long p, double *t, double *w, double *work)
       if (i == j)
         w[m] = sum + eta;
       else
-        w[m] = sum + sqrt(eta) * w[m]; 
+        w[m] = sum + sqrt(eta) * w[m];
     }
   }
 
@@ -711,11 +779,11 @@ void WishartRandom (long n, long p, double *t, double *w, double *work)
 
       for (k = j, k3 = j; k <= i; k3 = k3 + (++k))
         sum = sum + t[k1+k] * w[k2+k3];
-      
+
       work[m] = sum;
     }
   }
-  
+
   for (i = 0, m = 0, k1 = 0; i < p; i++, k1 = k1 + p) {
     for (j = 0, k2 = 0; j <= i; k2 = k2 + (++j), m++) {
       for (k = 0, sum = 0.0; k <= j; k++)
@@ -728,7 +796,7 @@ void WishartRandom (long n, long p, double *t, double *w, double *work)
 } /* WishartRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    UniformRandom
 
    returns a variate that is uniformly distributed on the interval [a,b].
@@ -737,12 +805,17 @@ void WishartRandom (long n, long p, double *t, double *w, double *work)
 double UniformRandom (double a, double b)
 {
 
-  return (Randoms()*(b - a) + a);
+  if (b < a) {
+    printf ("Error: bad range a for uniform variate - Exiting\n\n");
+    exit (0);
+  }
+
+  return (Randoms() * (b - a) + a);
 
 } /* UniformRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    LogUniformRandom
 
    returns a variate that is log-uniformly distributed on the interval
@@ -752,17 +825,22 @@ double UniformRandom (double a, double b)
 double LogUniformRandom (double a, double b)
 {
 
-  return ( a*pow(b/a, Randoms()) );
+  if (b < a) {
+    printf ("Error: bad range a for uniform variate - Exiting\n\n");
+    exit (0);
+  }
+
+  return ( a * pow(b/a, Randoms()) );
 
 } /* LogUniformRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    BetaRandom
 
    returns a variate that is Beta distributed on the interval [a,b]
    with shape parameters alpha and beta.
-   
+
    The Beta function has two shaping parameters, alpha and beta.
    Setting these parameters to 1.5 and 1.5 yields a normal-like
    distribution, but without tails. If alpha and beta are equal to
@@ -786,12 +864,15 @@ double LogUniformRandom (double a, double b)
    - J.H.Maindonald, Statistical Computation, John Wiley and Sons,
      1984, p 370.
 */
-/* TEMPORARILY PUT BACK THE a b alpha beta stupid order to conform to
-   simmonte.c call Frederic 23/8/96 */
-double BetaRandom (double a, double b, double alpha, double beta)
+double BetaRandom (double alpha, double beta, double a, double b)
 {
   double u1, u2, w;
 
+  if (b <= a || alpha <= 0 || beta <= 0) {
+    printf ("Error: bad shape or range for a beta variate - Exiting\n\n");
+    exit (0);
+  }
+
   if ((alpha < 1) && (beta < 1))
     /* use rejection */
     do {
@@ -804,7 +885,7 @@ double BetaRandom (double a, double b, double alpha, double beta)
       w = u1 + u2;
 
     } while (w > 1.0);
-  
+
   else {
     /* use relation to Gamma */
     u1 = GammaRandom(alpha);
@@ -817,11 +898,11 @@ double BetaRandom (double a, double b, double alpha, double beta)
 } /* BetaRandom */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    Utility functions
 */
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    erfc
 
    the error function of z (Press et al. Numerical Recipes)
@@ -840,41 +921,52 @@ double erfc(double x)
 }
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    CDFNormal
 
    the probability for [-inf;z] under the normal distribution
 */
 
-double CDFNormal(double z)
+double CDFNormal (double z)
 {
   return ( 0.5 * (2 - erfc(z/sqrt(2))) );
 }
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    lnDFNormal
    the log of the normal density function
    0.9189385332046 is log(sqrt(2*PI))
 */
 
-double lnDFNormal(double x, double mu, double sd)
+double lnDFNormal (double x, double mu, double sd)
 {
-  return ( -0.9189385332046 - log(sd) - 0.5 * pow((mu - x)/sd, 2) );
+  if (sd <= 0.0) {
+    printf ("Error: negative or null SD in lnDFNormal\n");
+    exit (0);
+  }
+
+  return ( -0.9189385332046 - log (sd) - 0.5 * pow ((mu - x)/sd, 2) );
 }
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    lnGamma
 
    A function to return the natural log of the Gamma function of x.
    From Press et al., Numerical Recipes.
+   It can be used to compute factorials since ln(n!) = lnGamma(n + 1)
 */
 
 double lnGamma (double x)
 {
   double tmp, ser;
 
+  if (x <= 0.0) {
+    printf ("Error: negative or null parameter for lnGamma function\n");
+    exit (0);
+  }
+  
   ser = 1.000000000190015 +
         76.18009172947146   /  x      -
         86.50532032141677   / (x + 1) +
@@ -884,12 +976,12 @@ double lnGamma (double x)
         5.39523938495E-6    / (x + 5);
 
   tmp = x + 4.5;
-  return -tmp + (x - 0.5) * log(tmp) + log(2.50662827465 * ser);
+  return -tmp + (x - 0.5) * log (tmp) + log (2.50662827465 * ser);
 
 } /* lnGamma */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    InterpolateX
 
    Do a linear interpolation to return x
@@ -904,7 +996,7 @@ double InterpolateX (double rgX[], double rgY[], long lLower, double dY)
 } /* InterpolateX */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    CalcCumulative
 
    Approximates to an iOrder the cumulative distribution rg_Cdf
@@ -951,7 +1043,7 @@ void CalcCumulative (long cDim, double *rg_x, double *rg_pdf,
 } /* CalcCumulative */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    PiecewiseVariate
 
    Returns a variate drawn by tabulated inversion from the
@@ -1000,7 +1092,7 @@ double PiecewiseVariate (long cDim, double rg_x[], double rg_pdf[],
   /* If we are exactly on a cumulative data point (unlikely),
      the value of the pdf is known and the variate is a grid point.
    */
-  if (lUpper == lLower)    {
+  if (lUpper == lLower) {
     dValPdf = rg_pdf[lLower];
     dPWVariate = rg_x[lLower];
   }
@@ -1058,6 +1150,8 @@ double PiecewiseVariate (long cDim, double rg_x[], double rg_pdf[],
     break;
 
     default:
+      dValPdf = 0;
+      dPWVariate = 0;
       assert(0);
       break;
 
@@ -1070,7 +1164,7 @@ double PiecewiseVariate (long cDim, double rg_x[], double rg_pdf[],
 } /* PiecewiseVariate */
 
 
-/* ----------------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
    PiecewiseRandom
                                                   __
    returns a variate that is distributed along a /  \ shaped distribution.
