@@ -3,10 +3,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.sim.c
-    Revision:  1.27
-        Date:  23 Jan 1996
-     Modtime:  09:13:01
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -25,7 +25,7 @@
 
 #include "yourcode.h"
 #include "getopt.h"
-#include "gibbs.h"
+#include "mh.h"
 #include "lexerr.h"
 #include "lsodes.h"
 #include "mac.h"
@@ -38,7 +38,7 @@
 #include "strutil.h"
 
 
-static char vszVersion[] = "v3.6"; /* Version of program */
+static char vszVersion[] = "v4.0"; /* Version of program */
 
 
 /* -----------------------------------------------------------------------------
@@ -63,6 +63,55 @@ void CorrectInputToTransition (PEXPERIMENT pexp, PDOUBLE pdTtrans)
 } /* CorrectInputToTransition */
 
 
+/* -----------------------------------------------------------------------------
+   Euler
+
+   Simple Euler integrator.
+
+*/
+
+int Euler (long neq, double *y, double *t, double tout, double dTStep)
+{
+  static PDOUBLE rgdDeriv;
+  double dTmp_step;
+  long   i;
+
+  if (!(rgdDeriv))
+    if ( !(rgdDeriv = InitdVector (neq)))
+      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "Euler", NULL);
+
+  /* Iterate through time out to prescrebed output */
+  while (*t < tout) {
+
+    /* Compute derivatives at current time */
+    CalcDeriv (y, rgdDeriv, t);
+
+    /* Update time */
+    *t = *t + dTStep;
+
+    /* But do not exceed prescribed tout */
+    if (*t > tout) {
+      dTmp_step = tout - (*t - dTStep);
+      *t = tout;
+    }
+    else
+      dTmp_step = dTStep;
+
+    /* Update the state variables */
+    for (i = 0; i < neq; i++)
+      y[i] = y[i] + dTmp_step * rgdDeriv[i];
+
+    DoStep_by_Step();
+  }
+
+  /* Calculate the derivatives at t = tout for cleanliness */
+  CalcDeriv (y, rgdDeriv, t);
+
+  return (0);
+
+} /* Euler */
+
+
 /* -----------------------------------------------------------------------------
    FreeVarMod
 
@@ -122,10 +171,10 @@ void ModifyParms (PLIST plistParmMods)
 } /* ModifyParms */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    DoOneExperiment
 
-   Runs one experiment
+   Runs one experiment - return 1 on success and 0 in case of errors
 */
 
 int DoOneExperiment (PEXPERIMENT pexp)
@@ -138,12 +187,12 @@ int DoOneExperiment (PEXPERIMENT pexp)
   PMODELINFO pmod;  /* pointer to the current model info */
   PINTSPEC   pis;   /* pointer to the integrator specs */
 
-  if (!pexp) return 1;
+  if (!pexp) return 0;
 
   pmod = pexp->pmodelinfo;
   pis  = &(pexp->is);
 
-  if (!InitOutputs (pexp, &iOut, &dTout)) return 1;
+  if (!InitOutputs (pexp, &iOut, &dTout)) return 0;
 
   UpdateInputs (&pexp->dT0, &dTtrans); /* Resolve dependent inputs */
 
@@ -152,27 +201,40 @@ int DoOneExperiment (PEXPERIMENT pexp)
   /* set lsodes flag to 1 for first call */
   pexp->is.iDSFlag = 1;
 
+  /* Iterate to final time */
   while (pexp->dTime < pexp->dTfinal) {
 
-    /* Iterate to final time */
-
-    if(pis->iAlgo == IAL_LSODES ) { /* Lsodes is chosen */
+    /* If dynamics equations are defined */
+    if (pmod->nStates > 0) {
 
       /* the upper limit of integration dTup should be either dTout
          or dTtrans, whichever is smaller */
       dTup = (dTout < dTtrans) ? dTout : dTtrans;
 
-      lsodes_(&pmod->nStates, pmod->pdModelVars, &(pexp)->dTime,
-              &dTup, &pis->itol, &pis->dRtol, &pis->dAtol,
-              &pis->itask, &pis->iDSFlag, &pis->iopt, pis->rwork,
-              &pis->lrw, pis->iwork, &pis->liw, &pis->iMf);
+      if (pis->iAlgo == IAL_LSODES) { /* Lsodes algorithm */
 
-      /* try to correct if bad return flag - FB 17 oct 96, 20 nov 96 */
-      if (pis->iDSFlag < 0) {
-          /* error conditions returned by lsodes - 
-             we cannot guarantee the accuracy of the results */
-          return (1);
+        lsodes_(&pmod->nStates, pmod->pdModelVars, &(pexp)->dTime,
+                &dTup, &pis->itol, &pis->dRtol, &pis->dAtol,
+                &pis->itask, &pis->iDSFlag, &pis->iopt, pis->rwork,
+                &pis->lrw, pis->iwork, &pis->liw, &pis->iMf);
+
+        /* Handle error returns : FB 25 nov 96 : */
+        if (pis->iDSFlag < 0) {
+          /* We cannot guarantee the accuracy of the results, exit the routine
+             with an error flag */
+          return (0);
+        }
       }
+      else {
+        if (pis->iAlgo == IAL_EULER) { /* Euler algorithm */
+          Euler (pmod->nStates, pmod->pdModelVars, &(pexp)->dTime, dTup,
+                 pis->dTStep);
+        }
+      }
+    }
+    else {
+      /* We still need to advance the time */
+      pexp->dTime = (dTout < dTtrans) ? dTout : dTtrans;
     }
 
     if (dTtrans <= dTout) {
@@ -185,7 +247,7 @@ int DoOneExperiment (PEXPERIMENT pexp)
           pis->iDSFlag = 1;
         }
       }
-      else { 
+      else {
         /* dTtrans == dTout */
         if (dTtrans < pexp->dTfinal) {
           CorrectInputToTransition (pexp, &dTtrans);
@@ -203,37 +265,40 @@ int DoOneExperiment (PEXPERIMENT pexp)
 
   } /* while dTime < final time */
 
-  if( !(pis->iAlgo == IAL_LSODES) ) {
-    printf ("Accessing disabled Gear-related code in sim.c - Exiting 4...\n");
-    exit (0);
-  }
-
-  return 0;
+  /* success */
+  return 1;
 
 } /* DoOneExperiment */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    DoOneNormalExp
 
    Does one AT_DEFAULTSIM simulation.
+
+   Return 1 on success and 0 in case of failure
 */
 
-void DoOneNormalExp (PANALYSIS panal, PEXPERIMENT pexp)
+int DoOneNormalExp (PANALYSIS panal, PEXPERIMENT pexp)
 {
   printf (" %d", pexp->iExp); /* Show what experiment it is */
 
   InitModel ();
   ModifyParms (panal->expGlobal.plistParmMods); /* Global modifications */
   ModifyParms (pexp->plistParmMods); /* Mods for this experiment */
-  DoOneExperiment (pexp);
+  if (!DoOneExperiment (pexp)) {
+    /* Error */
+    return 0;
+  }
 
   printf ("\n");
 
+  return (1);
+
 } /* DoOneNormalExp */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    DoOneMCExp
 
    Does one AT_MONTECARLO simulation.
@@ -259,9 +324,11 @@ void DoOneNormalExp (PANALYSIS panal, PEXPERIMENT pexp)
    currently won't happen unless the model is init'd.  Maybe create a
    ResetInputs() with starting time which will do the funky stuff
    done by the global variable right now.
+
+   Return 1 on success and 0 in case of failure
 */
 
-void DoOneMCExp (PANALYSIS panal, PEXPERIMENT pexp)
+int DoOneMCExp (PANALYSIS panal, PEXPERIMENT pexp)
 {
   register MONTECARLO *pmc = &panal->mc;
 
@@ -269,7 +336,12 @@ void DoOneMCExp (PANALYSIS panal, PEXPERIMENT pexp)
   ModifyParms (panal->expGlobal.plistParmMods); /* Global modifications */
   SetParms (pmc->nParms, pmc->rghvar, pmc->rgdParms); /* MC mods */
   ModifyParms (pexp->plistParmMods); /* Mods for this experiment */
-  DoOneExperiment (pexp);
+  if (!DoOneExperiment (pexp)) {
+    /* Error */
+    return 0;
+  }
+
+  return (1);
 
 } /* DoOneMCExp */
 
@@ -289,14 +361,18 @@ void DoNormal (PANALYSIS panal)
        (nExps > 1 ? 's' : ' '));
 
   for (i = 0; i < nExps; i++) {
-    DoOneNormalExp (panal, panal->rgpExps[i]);
-    WriteNormalOutput (panal, panal->rgpExps[i]);
+    if (DoOneNormalExp (panal, panal->rgpExps[i])) {
+      /* if successfull write out the results */
+      WriteNormalOutput (panal, panal->rgpExps[i]);
+    }
+    else
+      printf ("Warning: Integration failed - No output generated\n");
   }
 
 } /* DoNormal */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    DoMonteCarlo
 
    Does a Monte Carlo analysis or a Set Points analysis.  The latter is
@@ -321,9 +397,9 @@ void DoNormal (PANALYSIS panal)
 void DoMonteCarlo (PANALYSIS panal)
 {
   int nExps = panal->expGlobal.iExp;
-  int nRuns = panal->mc.nRuns;
+  long nRuns = panal->mc.nRuns;
   MCPREDOUT mcpredout;
-  BOOL bNotDone; /* Not finished with analysis */
+  BOOL bOK = FALSE, bNotDone; /* Not finished with analysis */
   int i;
 
   mcpredout.pred = NULL;
@@ -334,7 +410,7 @@ void DoMonteCarlo (PANALYSIS panal)
   /* if cannot open files, Abort */
   if (OpenMCFiles (panal)) exit(0);
 
-  printf ("\nDoing analysis - %d %s run%c... %d experiment%c%s\n",
+  printf ("\nDoing analysis - %ld %s run%c... %d experiment%c%s\n",
           nRuns,
           (panal->iType == AT_MONTECARLO ? "Monte Carlo" : "Set point"),
           (nRuns != 1 ? 's' : ' '),
@@ -353,12 +429,21 @@ void DoMonteCarlo (PANALYSIS panal)
 
     if (bNotDone) {
       /* Do analysis if not finished */
-      for (i = 0; i < nExps; i++)  /* Do all experiments */
-        DoOneMCExp (panal, panal->rgpExps[i]);
+      for (i = 0; i < nExps; i++) { /* Do all experiments */
+        bOK = DoOneMCExp (panal, panal->rgpExps[i]);
+        if (!bOK) break;
+      }
 
-      TransformPred (panal, &mcpredout); /* transform output run */
-      WriteMCOutput (panal, &mcpredout);
-    } /* if */
+      if (bOK) {
+        /* If successful write results */
+        TransformPred (panal, &mcpredout); /* transform output run */
+        WriteMCOutput (panal, &mcpredout);
+      }
+      else
+        printf ("Warning: "
+                "Integration failed on iteration %ld, experiment %d:"
+                "         No output generated\n", panal->mc.lRun, i);
+    } /* if bNotDone */
 
     panal->mc.lRun++; /* Next run */
     if (nRuns) /* If a number of runs spec'd... */
@@ -392,6 +477,7 @@ void DoAnalysis (PANALYSIS panal)
       break;
 
     case AT_GIBBS:
+    case AT_MCMC:
       DoMarkov (panal);
       break;
 
@@ -477,13 +563,15 @@ void PrepAnalysis (PANALYSIS panal)
 */
 
 void CheckDependencies (PANALYSIS panal) {
-  MONTECARLO *pmc = &panal->mc;
+/*  MONTECARLO *pmc = &panal->mc;
   MCVAR *pmcvar;
   int n, m, l;
   char *level, lv, prevLevel, varParm;
   char s[4];
   BOOL changed;
-  for(n = 0; n < MC_DEPLEVELS; ++n) {
+  pmc->nLevels = 0;
+  for(n = 0; n < MAX_LEVELS; ++n) {
+    ++pmc->nLevels;
     changed = FALSE;
     for(m = 0; m < pmc->nParms; ++m) {
       for(l = 0; l < pmc->nParms; ++l) {
@@ -496,20 +584,19 @@ void CheckDependencies (PANALYSIS panal) {
            ((varParm & 4) && (pmcvar->uMin.hvar == pmc->rgpMCVar[m]->hvar)) ||
            ((varParm & 8) && (pmcvar->uMax.hvar == pmc->rgpMCVar[m]->hvar))) {
             prevLevel = *level;
-            if((lv = pmc->rgpMCVar[m]->cLevel + 1) == MC_DEPLEVELS) {
+            if((lv = pmc->rgpMCVar[m]->cLevel + 1) == MAX_LEVELS) {
               sprintf(s, "%d", lv);
               ReportError(NULL, RE_TOOMANYLEVELS | RE_FATAL, s, NULL);
             }
             if(lv > prevLevel) {
               *level = lv;
-              fprintf(stderr, "level of %d changed to %d\n", l, *level);
               changed = TRUE;
             }
         }
       }
     }
     if(!changed) break;
-  }
+  }*/
 } /* CheckDependencies */
 
 /* Get the command line argument stuff */
@@ -549,16 +636,25 @@ void PromptFilenames (PSTR *pszFileIn, PSTR *pszFileOut)
   *pszFileOut = (PSTR) calloc (1, 80);
 
   printf ("Input filename? ");
+#ifdef _MACOS_
+  scanf ("%[^:\f\r\v\n]", *pszFileIn);
+  getchar();
+#else
   gets (*pszFileIn);
   *pszFileIn = strtok (*pszFileIn, " \t");
+#endif
 
   if (!(*pszFileIn)) /* Nothing entered, quit */
     return;
 
   if ((*pszFileIn)[0]) { /* Input file specified */
     printf ("Output filename? ");
+#ifdef _MACOS_
+    scanf ("%[^:\f\r\v\n]", *pszFileOut);
+#else
     gets (*pszFileOut);
     *pszFileOut = strtok (*pszFileOut, " \t");
+#endif
   }
 
   if (!(*pszFileOut) || !(*pszFileOut)[0]) { /* If no output specified */
@@ -685,6 +781,12 @@ BOOL GetCmdLineArgs (int cArg,
         case 'D':
           printf (">> Debug mode: Using option '%s'\n", optarg);
           /* Can setup to run with certain debug flags */
+          if(!strcmp(optarg, "d"))
+            panal->bDependents = TRUE;
+          else if(!strcmp(optarg, "i"))
+            panal->bPrintIter = TRUE;
+          else if(!strcmp(optarg, "p"))
+            panal->bParams = TRUE;
           break;
 
         case 'n':
@@ -723,9 +825,8 @@ BOOL GetCmdLineArgs (int cArg,
         break;
 
       case 5: /* Input and 2 output files spec'd */
-        *pszFileIn = rgszArg[optind];
-        panal->mc.szMCPassFilename = rgszArg[optind + 1];
-        panal->mc.szMCFailFilename = rgszArg[optind + 2];
+        printf ("Error: the 2 output files option is now obsolete - "
+                "Exiting\n\n");
         break;
 
       case 2: /* Output and input file specificed */
@@ -786,13 +887,14 @@ int main (int nArg, char *const *rgszArg)
 
 #ifdef _MACOS_
 
-  InitMacintosh ();
-  while (1) HandleEvent();
+/*InitMacintosh ();
+  while (1) HandleEvent(); */
 
 #endif
 
   AnnounceProgram ();
 
+
   if (!panal)
     ReportError (NULL, RE_OUTOFMEM | RE_FATAL,
                  "ANALYSIS specification too large", NULL);
@@ -803,7 +905,8 @@ int main (int nArg, char *const *rgszArg)
 
   /* Define the output file as the global experiment default  */
   panal->expGlobal.os.szOutfilename = szFileOut;
-  panal->expGlobal.os.bCommandLineSpec = (BOOL) szFileOut;  /* Flag */
+  szFileOut == NULL ? (panal->expGlobal.os.bCommandLineSpec = FALSE) :
+                      (panal->expGlobal.os.bCommandLineSpec = TRUE);
 
   if (!InitBuffer (&ibIn, szFileIn))
     ReportError (&ibIn, RE_INIT | RE_FATAL, "ReadInput", NULL);
@@ -816,6 +919,7 @@ int main (int nArg, char *const *rgszArg)
     CheckDependencies (panal);
     DoAnalysis (panal);
   } /* if */
+  FreeLevels(panal);
 
 #ifdef _MACOS_
   printf ("Done.(Hit Return)\n\n");
