@@ -6,9 +6,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.siminit.h
-    Revision:  1.5
-        Date:  13 Dec 1995
-     Modtime:  16:04:45
+    Revision:  1.6
+        Date:  23 Nov 1996
+     Modtime:  12:39:47
       Author:  @a
    -- SCCS  ---------
 
