@@ -3,10 +3,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.simmonte.c
-    Revision:  1.15
-        Date:  13 Jan 1996
-     Modtime:  15:55:36
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -26,37 +26,6 @@
 #include "strutil.h"
 #include "simmonte.h"
 
-/* -----------------------------------------------------------------------------
-   ModifyOneMCParm
-
-   Callback function for ModifyMCParms.
-*/
-
-int ModifyOneMCParm (PVOID pData, PVOID pInfo)
-{
-  PMCVAR  pMCVar = (PMCVAR) pData; /* Current parm variation */
-
-  SetVar (pMCVar->hvar, pMCVar->dVal);
-
-  return 0;
-
-} /* ModifyOneMCParm */
-
-
-/* -----------------------------------------------------------------------------
-   ModifyMCParms
-
-   Make Monte Carlo modifications defined in pMC->plistMCVars using a
-   callback function to traverse the list.
-*/
-
-void ModifyMCParms (PMONTECARLO pMC)
-{
-  assert (pMC->plistMCVars);
-  ForAllList (pMC->plistMCVars, ModifyOneMCParm, NULL);
-
-} /* ModifyMCParms */
-
 
 /* -----------------------------------------------------------------------------
    SetParms
@@ -75,84 +44,97 @@ void SetParms (long cParms, HVAR *rghvar, double *rgdParm)
 } /* SetParms */
 
 
-/* -----------------------------------------------------------------------------
-   SetParmsLog
-
-   sets the parameters in the rghvar array to the log-transformed
-   values in the rgdParm array.
-*/
-
-void SetParmsLog (long cParms, HVAR *rghvar, double *rgdParm)
-{
-  long i;
-
-  for (i = 0; i < cParms; i++)
-    SetVar (rghvar[i], exp(rgdParm[i]));
-
-} /* SetParmsLog */
-
-
 /* -----------------------------------------------------------------------------
    CalculateOneMCParm
 
    Callback function for CalculateMCParms.
 */
 
-int CalculateOneMCParm (PMCVAR pMCVar, PMONTECARLO pMC)
+int CalculateOneMCParm (PMCVAR pMCVar)
 {
-  double  dRand = 0.0;
+  double dParm1, dParm2, dMin, dMax;
 
-  if (pMC && !pMC->bIndependent)
-    dRand = pMC->dRandTemp; /* Use same value for each parm */
-
-  else if (pMCVar->iType == MCV_UNIFORM || pMCVar->iType == MCV_LOGUNIFORM)
-    dRand = Randoms(); /* Randomize for each parameter */
-
-    /* Set variable randomly according to selected distribution */
+  /* Check pMCVar for dependencies */
+  dParm1 = *(pMCVar->pdParm[0]);
+  dParm2 = *(pMCVar->pdParm[1]);
+  dMin   = *(pMCVar->pdParm[2]);
+  dMax   = *(pMCVar->pdParm[3]);
 
+  /* Set variable randomly according to selected distribution */
   switch (pMCVar->iType) {
 
     default:
-    case MCV_UNIFORM: /* Uniform distribution */
-      pMCVar->dVal = dRand * (pMCVar->uMax.dval - pMCVar->uMin.dval) +
-                     pMCVar->uMin.dval;
+    case MCV_UNIFORM:
+      pMCVar->dVal = UniformRandom (dMin, dMax);
       break;
 
-    case MCV_LOGUNIFORM: /* Log uniform distribution */
-      pMCVar->dVal = pMCVar->uMin.dval *
-                     pow (pMCVar->uMax.dval / pMCVar->uMin.dval, dRand);
+    case MCV_LOGUNIFORM:
+      pMCVar->dVal = LogUniformRandom (dMin, dMax);
       break;
 
-    case MCV_NORMAL: /* Normal distrib, mean, stdev */
-      pMCVar->dVal = NormalRandom (pMCVar->uParm1.dval, pMCVar->uParm2.dval);
+    case MCV_BETA:
+      pMCVar->dVal = BetaRandom (dParm1, dParm2, dMin, dMax);
       break;
 
-    case MCV_LOGNORMAL: /* Log normal distrib, mean, stdev */
-      pMCVar->dVal = LogNormalRandom (pMCVar->uParm1.dval,
-                                      pMCVar->uParm2.dval);
+    case MCV_NORMAL:
+      pMCVar->dVal = NormalRandom (dParm1, dParm2);
       break;
 
-    case MCV_PIECEWISE: /* Piecewise distrib, min, max, val1, val2 */
-      pMCVar->dVal = PiecewiseRandom (pMCVar->uMin.dval, pMCVar->uParm1.dval,
-                                      pMCVar->uParm2.dval, pMCVar->uMax.dval);
-      break;
-
-    case MCV_BETA: /* Beta distribution */
-      pMCVar->dVal = BetaRandom (pMCVar->uMin.dval, pMCVar->uMax.dval,
-                                 pMCVar->uParm1.dval, pMCVar->uParm2.dval);
+    case MCV_LOGNORMAL:
+      pMCVar->dVal = LogNormalRandom (dParm1, dParm2);
       break;
 
     case MCV_TRUNCNORMAL:
-      pMCVar->dVal = TruncNormalRandom (pMCVar->uParm1.dval,
-                                        pMCVar->uParm2.dval,
-                                        pMCVar->uMin.dval, pMCVar->uMax.dval);
+      pMCVar->dVal = TruncNormalRandom (dParm1, dParm2, dMin, dMax);
       break;
 
     case MCV_TRUNCLOGNORMAL:
-      pMCVar->dVal = TruncLogNormalRandom (pMCVar->uParm1.dval,
-                                           pMCVar->uParm2.dval,
-                                           pMCVar->uMin.dval,
-                                           pMCVar->uMax.dval);
+      pMCVar->dVal = TruncLogNormalRandom (dParm1, dParm2, dMin, dMax);
+      break;
+
+    case MCV_NORMALV:
+      pMCVar->dVal = NormalRandom (dParm1, sqrt (dParm2));
+      break;
+
+    case MCV_LOGNORMALV:
+      pMCVar->dVal = LogNormalRandom (dParm1, exp(sqrt(log(dParm2))));
+      break;
+
+    case MCV_TRUNCNORMALV:
+      pMCVar->dVal = TruncNormalRandom (dParm1, sqrt (dParm2), dMin, dMax);
+      break;
+
+    case MCV_TRUNCLOGNORMALV:
+      pMCVar->dVal = TruncLogNormalRandom (dParm1, exp(sqrt(log(dParm2))),
+                                           dMin, dMax);
+      break;
+
+    case MCV_CHI2:
+      pMCVar->dVal = Chi2Random (dParm1);
+      break;
+
+    case MCV_BINOMIAL:
+      pMCVar->dVal = BinomialRandom (dParm1, (long) dParm2);
+      break;
+
+    case MCV_PIECEWISE:
+      pMCVar->dVal = PiecewiseRandom (dMin, dParm1, dParm2, dMax);
+      break;
+
+    case MCV_EXPONENTIAL:
+      pMCVar->dVal = ExpRandom (dParm1);
+      break;
+
+    case MCV_GGAMMA:
+      pMCVar->dVal = GGammaRandom (dParm1, dParm2);
+      break;
+
+    case MCV_INVGGAMMA:
+      pMCVar->dVal = InvGGammaRandom (dParm1, dParm2);
+      break;
+
+    case MCV_POISSON:
+      pMCVar->dVal = PoissonRandom (dParm1);
       break;
 
   } /* switch */
@@ -183,11 +165,8 @@ void CalcMCParms (PMONTECARLO pMC, double rgParms[], long iStart)
   if (!rgParms)
     rgParms = pMC->rgdParms; /* Put them in the usual place */
 
-  if (!pMC->bIndependent) /* If not independent, use one */
-    pMC->dRandTemp = Randoms(); /* draw for all parameters.    */
-
   for (i = iStart; i < pMC->nParms; i++) {
-    CalculateOneMCParm (pMC->rgpMCVar[i], pMC);
+    CalculateOneMCParm (pMC->rgpMCVar[i]);
     rgParms[i] = pMC->rgpMCVar[i]->dVal;
   } /* for */
 
